<?php

namespace App\Http\Controllers;

use DB;
use Log;
use App\Users;
use Illuminate\Http\Request;
use App\Http\ResponseHelper;
use App\Jobs\CreateShopifyCustomerJob;
use App\Jobs\TagUserJob;

class QuizController extends Controller
{

    private $helper;

    public function __construct()
    {
        $this->helper = new ResponseHelper();
    }

    public function storeQuiz(Request $request)
    {
        $page         = $request->input('page');
        $sessionID    = $request->input('sessionID');
        $question     = $request->input('question');
        $questionID   = $request->input('questionID');
        $responses    = $request->input('responses');
        $responseIds  = $request->input('responseIDs');
        $responseKeys = $request->input('responseKeys');
        
        if(empty($sessionID)) {
            
            // error
            $returnData = [
                'error' => 'Invalid Session'
            ];

            return $this->helper->returnData(600, "error", $returnData );
        }
        
        if( $page == 'close')
        {
            $updateQuizUser = DB::table('quiz_user')
                ->where('session_id', $sessionID)
                ->update(['status' => 1]);

            $userResponse = DB::select('
                select 
                    qm.id,
                    qm.question,
                    qr.response,
                    qr.response_key,
                    ur.response_text
                from 
                    user_responses as ur
                join 
                    question_master as qm 
                on 
                    qm.id = ur.qn_id
                left join 
                    question_responses as qr
                on 
                    ur.response_id = qr.id
                where 
                    session_id = ?', [$sessionID]);

            $dataResult = [];
            $details = [];
                   
                    
            if(!empty($userResponse))
            {
                
                foreach($userResponse as $response)
                {
                    $questionId   = $response->id;
                    $questionText = $response->question;
                    $answerText   = $response->response;
                    $answerKey    = $response->response_key;
                    $responseText = $response->response_text;
                    
                    if (!isset($details[$questionId])){
                        $details[$questionId] = [];
                        $details[$questionId]['QuestionNo'] = $questionId;
                    }
                    
                    $details[$questionId]['AnswerText'][] = empty($answerKey) ? $responseText : $answerKey;             
                }
                $questionDetails = array_values($details);
                $dataResult['SurveyAnswers']['SessionId'] = $sessionID;
                $dataResult['SurveyAnswers']['Questions'] = $questionDetails;
            }

            $returnData = $dataResult;

            // CALL to Recommendation API - TODO
            
            // $recommendation = $this->callRecommendation($returnData);

            $recommendation = '{
                "SessionID":"Session1",
                "Email":"sdrolet@pharmavite.net",
                "Firstname":"Stephane",
                "SexProfile":"Male",
                "Sex":"M",
                "Age":50,
                "ZipCode":"55101",
                "Latitude":44.951463,
                "Longitude":-93.090745,
                "City":"Saint Paul",
                "State":"MN",
                "Allergens":{"Wheat":false,"Fish":true,"Shellfish":false,"Soy":false,"Milk":false},
                "Deficiencies":{"VitaminD":true,"VitaminB12":false,"Iron":true,"Magnesium":false},
                "Recommendation":[{"Name":"Lutein","VariantId":"17751228514375","Bulk":"985","Quantity":1},{"Name":"Mens Multi","VariantId":"17751229759559","Bulk":"A064AH","Quantity":1},{"Name":"Vitamin C 500 mg","VariantId":"17751230709831","Bulk":"A101AH","Quantity":5},{"Name":"Omega-3 720 mg","VariantId":"18048155058247","Bulk":"A333AB","Quantity":1},{"Name":"Omega-3 + Vitamin D","VariantId":"17751229956167","Bulk":"A555AC","Quantity":4},{"Name":"Vitamin D3 125 mcg","VariantId":"18048200900679","Bulk":"A566AD","Quantity":1},{"Name":"Magnesium Oxide","VariantId":"17751230414919","Bulk":"A806AB","Quantity":1}],
                "BuzzWordsTips":{"BuzzWords1":["yoga-posing","wave-catching","green-thumbing"],"BuzzWords2":["clean-eating"],"BuzzWords3":["friends-and-family-loving"],"Tips1":["Let’s namaste the heck out of this day.","Don’t wait for the perfect wave. Just surf.","Thoughts are like waves. We can’t stop them from coming but we can choose which ones to surf.","Putting your hands in the Earth connects you back to it."]},
                "North37":true,
                "ProstateInterest":true,
                "PrenatalInterest":false,
                "PostnatalInterest":false,
                "HealthyMentrualCycleInterest":false,
                "PerimenopauseInterest":false,
                "MenopauseInterest":false,
                "PlanningPregnancy":false,
                "Pregnant":false,
                "Breastfeeding":false,
                "TakingMedication":false,
                "BloodClottingIssue":false
            }';

            $recommendationObj = json_decode($recommendation);

            $lastInsertedId = DB::table('recommendations')->insertGetId(
                [
                    'session_id' => $recommendationObj->SessionID,  
                    'email' => $recommendationObj->Email, 
                    'first_name' => $recommendationObj->Firstname,
                    'sex_profile' => $recommendationObj->SexProfile,
                    'sex' => $recommendationObj->Sex,
                    'age' => $recommendationObj->Age,
                    'zipcode' => $recommendationObj->ZipCode,
                    'latitude' => $recommendationObj->Latitude,
                    'longitude' => $recommendationObj->Longitude,
                    'city' => $recommendationObj->City,
                    'state' => $recommendationObj->State,
                    'allergens' => json_encode($recommendationObj->Allergens),
                    'deficiencies' => json_encode($recommendationObj->Deficiencies),
                    'recommendation' => json_encode($recommendationObj->Recommendation),
                    'buzz_words' => json_encode($recommendationObj->BuzzWordsTips),
                    'north37' => $recommendationObj->North37,
                    'prostate_interest' => $recommendationObj->ProstateInterest,
                    'prenatal_interest' => $recommendationObj->PrenatalInterest,
                    'postnatal_interest' => $recommendationObj->PostnatalInterest,
                    'healthy_interest' => $recommendationObj->HealthyMentrualCycleInterest,
                    'perimenopause_interest' => $recommendationObj->PerimenopauseInterest,
                    'menopause_interest' => $recommendationObj->MenopauseInterest,
                    'planning_pregnancy' => $recommendationObj->PlanningPregnancy,
                    'pregnant' => $recommendationObj->Pregnant,
                    'breastfeeding' => $recommendationObj->Breastfeeding,
                    'taking_medication' => $recommendationObj->TakingMedication,
                    'blood_clotting_issue' => $recommendationObj->BloodClottingIssue,
        
                ]
            );

            if(empty($lastInsertedId))
            {
                $returnData = [
                    'error' => 'Insertion Failed'
                ];
    
                return $this->helper->returnData(600, "error", $returnData );
            }

            // Tag User 
            $tagUser = dispatch(new TagUserJob($recommendation)); 

            return $this->helper->returnData(200, "success", json_decode($recommendation) );

        }

        if(empty($questionID)) {
            // error

            $returnData = [
                'error' => 'Quiz not found'
            ];

            return $this->helper->returnData(600, "error", $returnData );
        }

        // To delete the already existing question and update the new record 
        $deleteQuizDetails = DB::table('user_responses')
                    ->where([
                        ['session_id', '=', $sessionID],
                        ['qn_id', '>=', $questionID]
                    ])->delete();

        if(!empty($responseIds))
        {
            $lastInsertedId = null;
            foreach($responseIds as $responseId)
            {
                $lastInsertedId = DB::table('user_responses')->insertGetId(
                    [
                        'session_id' => $sessionID, 
                        'qn_id' => $questionID, 
                        'response_id' => $responseId, 
                        // 'response_text' => null, 
                        'deleted' => 0,
                        'status' => 0
                    ]
                );
            }

            if(empty($lastInsertedId))
            {
                $returnData = [
                    'error' => 'Insertion Failed'
                ];
    
                return $this->helper->returnData(600, "error", $returnData );
            }

            $returnData = [
                'stored' => true,
                'message' => 'success'
            ];            

        }
        else
        {
        
            $lastInsertedId = DB::table('user_responses')->insertGetId(
                [
                    'session_id' => $sessionID, 'qn_id' => $questionID, 
                    'response_id' => null, 
                    'response_text' => $responses[0], 'deleted' => 0,
                    'status' => 0
                ]
            );

            if(empty($lastInsertedId))
            {
                $returnData = [
                    'error' => 'Insertion Failed'
                ];
    
                return $this->helper->returnData(600, "error", $returnData );
            }

            $returnData = [
                'stored' => true,
                'message' => 'success'
            ];

        }

        if($page == 'email')
        {
            $email = isset($responses[0]) ? $responses[0] : '';

            if(empty($email))
            {
                $returnData = [
                    'error' => 'Email is empty'
                ];
    
                return $this->helper->returnData(600, "error", $returnData );
            }

            $user = DB::table('users')->where('user_email', '=', $email)->get();

            if(!isset($user[0]->user_email))
            {
                // create
                $userName = '';
                $userId = DB::table('users')->insertGetId([
                    'user_email' => $email,
                    'status' => 1
                ]);

                $quizId = DB::table('quiz_user')->insertGetId([
                    'user_id' => $userId,
                    'session_id' => $sessionID,
                    'deleted' => 0,
                    'status' => 0
                ]);

                $userNameResponse =  DB::table('user_responses')->where([
                    ['session_id', '=', $sessionID],
                    ['qn_id', '=', '2']
                ])->get();

                if($userNameResponse[0])
                {
                    $userName = $userNameResponse[0]->response_text;
                }

                // Create customer in Shopify
                $result = dispatch(new CreateShopifyCustomerJob($email, $userName)); 

                $returnData = [
                    'stored' => true,
                    'message' => 'success'
                ];
            }
            else
            {
                $userId    = $user[0]->id;
                $userEmail = $user[0]->user_email;
                $quizId    = DB::table('quiz_user')->insertGetId([
                    'user_id' => $userId,
                    'session_id' => $sessionID,
                    'deleted' => 0,
                    'status' => 0
                ]);
                
                $returnData = [
                    'stored' => true,
                    'message' => 'success'
                ];
            } 

        }

        return $this->helper->returnData(200, "success", $returnData );
        
    }

    public function deleteQuiz(Request $request)
    {
        $quizId = $request->input('quizId');
        $email = $request->input('email');
        $userID = '';

        if(empty($quizId))
        {
            $returnData = [
                'error' => 'Invalid Quiz Id'
            ];

            return $this->helper->returnData(600, "error", $returnData );
            
        }

        $userData  = DB::table('users')->where('user_email', '=', $email)->get();
        if(isset($userData[0]->id))
        {
            $userID = $userData[0]->id;
        }
        else
        {
            $returnData = [
                'error' => 'Invalid Email Id'
            ];

            return $this->helper->returnData(600, "error", $returnData );
        }
        if(!empty($userID))
        {
            $quizIds = array();
            $quizzes = DB::table('quiz_user')->where('user_id', '=', $userID)->get();

            if(!empty($quizzes))
            {
                foreach($quizzes as $quiz)
                {
                    array_push($quizIds, $quiz->id);
                }
            }

            if(!in_array($quizId, $quizIds))
            {
                $returnData = [
                    'error' => 'Quiz not exist in the server'
                ];
    
                return $this->helper->returnData(600, "error", $returnData );
            }

            $updateQuiz = DB::table('quiz_user')
                ->where('id', $quizId)
                ->update(['deleted' => 1]);

            if($updateQuiz == 1)  // success
            {
                $returnData = [
                    'deleted' => true,
                    'message' => 'successfully deleted'
                ];

                return $this->helper->returnData(200, "success", $returnData );
            }
            else
            {
                $returnData = [
                    'error' => 'Quiz does not exist'
                ];

                return $this->helper->returnData(600, "error", $returnData );
            }  
        }
    }

    public function getQuizzes(Request $request)
    {

        $userEmail = $request->input('email');
        $records   = 6 ; //$request->input('records');
        $page      = empty($request->input('page')) ? 1 : $request->input('page');

        $userData  = DB::table('users')->where('user_email', '=', $userEmail)->get();
        if(isset($userData[0]->id))
        {
            $userID = $userData[0]->id;
        }
        else
        {
            $returnData = [
                'error' => 'Invalid Email Id'
            ];

            return $this->helper->returnData(600, "error", $returnData );
        }
        

        $userQuizCount = DB::table('quiz_user')->where('user_id', '=', $userID)
                            ->get()->count();
        
        $userQuizDetails = DB::table('quiz_user')
            ->where([
                    ['user_id', '=', $userID],
                    ['deleted', '=', '0']
                ])
            ->orderBy('id', 'asc')    
            ->limit($records)
            ->offset(($page-1)*$records)
            ->get();

        
        $quizDetailsArray = [];
        foreach($userQuizDetails as $userQuizDetail)
        {
            $quizId = $userQuizDetail->id;
            $createdAt = $userQuizDetail->created_at;
            $updatedAt = $userQuizDetail->updated_at;
            
            $temp = [];
            $temp['quizID']    = $quizId;
            $temp['startDate'] = $createdAt;
            $temp['endDate']   = $updatedAt;

            array_push($quizDetailsArray, $temp);
        }    

        if(($records * $page) < $userQuizCount)
        {
            $hasNext = true;
        }
        else{
            $hasNext = false;
        }

        $returnData = [
            'email' => $userEmail,
            'totalRecords' => $userQuizCount,
            'pageSize' => $records,
            'pageNumber' => $page,
            'hasNext' => $hasNext,
            'quizzes' => $quizDetailsArray
        ];

        return $this->helper->returnData(200, "success", $returnData );       

    }

    public function getQuizDetails(Request $request)
    {
        $quizId = $request->input('quizId');

        $results = DB::select('
            select 
                qm.id,
                qm.question,
                qr.response
            from 
                user_responses as ur
            join 
                question_master as qm 
            on 
                qm.id = ur.qn_id
            join 
                question_responses as qr
            on 
                ur.response_id = qr.id
            where 
                quiz_id = ?', [$quizId]);

        $dataResult = [];
        $details = [];
        
        foreach($results as $result)
        {
            $questionId   = $result->id;
            $questionText = $result->question;
            $answerText   = $result->response;

            $temp = [];
           
            if (!isset($details[$questionId])){
                $details[$questionId] = [];
                $details[$questionId]['QuestionNo']   = $questionId;
                $details[$questionId]['QuestionText'] = $questionText;
            }
            
            $details[$questionId]['AnswerText'][]= $answerText;
        }

        $questionDetails['Questions'] = $details;
        $dataResult['SessionId']      = $quizId;
        $dataResult['SurveyAnswers']  = $questionDetails;
        
        $returnData = $dataResult;

        return $this->helper->returnData(200, "success", $returnData );

    }


}
