<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Config;
use OhMyBrew\BasicShopifyAPI;
use DB;
use Log;


class TagUserJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;


    public $recommendation;
    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($recommendation)
    {
        $this->recommendation = $recommendation;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {

        $recommendation = json_decode($this->recommendation);
        $sessionID    = $recommendation->SessionId;
        $profile      = $recommendation->Profile;
        $pillRec      = $recommendation->PillRecommendation;
        $sex          = $profile->SexProfile;
        $allergens    = $profile->ListAllergens;
        $deficiencies = $profile->ListDeficiencies;

        $tag = '';
        $allergensTag = '';
        $deficiencyTag = '';

        foreach($allergens as $key => $data)
        {
            if($data == true)
            {
                $allergensTag.= $key;
            }
        }

        foreach($deficiencies as $key => $data)
        {
            if($data == false)
            {
                $deficiencyTag.= $key;
            }
        }

        foreach($pillRec as $value)
        {
            $bulk = $value->Bulk;
            $name = $value->Name;
            $quantity = $value->Quantity;

            $tag.= " Recommendation:".$bulk."||".$name."||".$quantity.",";

        }

        if(!empty($allergensTag))
        {
            $tag .= 'Allergen:'.$allergensTag.',';
        }
        
        if(!empty($deficiencyTag))
        {
            $tag .= 'Deficiencies:'.$deficiencyTag;
        }

        Log::info("SessionID => ".$sessionID. " - Tag => ".$tag);
        
        $api = new BasicShopifyAPI();

        $password   = Config::get('shopify.password');
        $shopDomain = Config::get('shopify.shopDomain');

        $api->setShop($shopDomain);
        $api->setAccessToken($password);

        $quizUser = DB::table('quiz_user')->where('session_id', '=', $sessionID)->get();

        if(isset($quizUser[0]->user_id))
        {
            $userId = $quizUser[0]->user_id;

            $userEmail = DB::table('users')->where('id', '=', $userId)->get();
            $emailId = ($userEmail[0]->user_email);

            $params = NULL;
    
            $result = $api->rest('GET', 
                htmlentities('/admin/customers/search.json?query=email:'.$emailId), $params); 

            if(!empty($result->body))
            {
                $customerId = $result->body->customers[0]->id;

                $params = array(
                    'customer' => array(
                        'id' => $customerId,
                        'tags' => $tag   
                    )
                );

                Log::info($params);
        
                $result = $api->rest('PUT', '/admin/customers/'.$customerId.'.json', $params); 
                
            }     

        }

    }
}
